require "mini_parser"

describe JsDuck::Aggregator do
  def parse(string)
    Helper::MiniParser.parse(string, {:ext4_events => true})
  end

  describe "event inside Ext.define get extra parameter" do
    let(:event) do
      parse(<<-EOF)["Blah"][:members][0]
        /** */
        Ext.define("Blah", {
            /**
             * @event click
             * @param {Number} foo
             * @param {String} bar
             */
        });
      EOF
    end

    it "added to end" do
      event[:params].length.should == 3
    end

    it "named eOpts" do
      event[:params][2][:name].should == "eOpts"
    end

    it "of type Object" do
      event[:params][2][:type].should == "Object"
    end

    it "with standard description" do
      event[:params][2][:doc].should =~ /The options object passed to.*addListener/
    end

    it "with special :ext4event flag" do
      event[:params][2][:ext4_auto_param].should == true
    end
  end

  describe "When some class defined with Ext.define" do
    let(:events) do
      parse(<<-EOF)["Foo"][:members]
        /** @class Foo */
            /**
             * @event click
             * @param {Number} foo
             */
            /**
             * @event touch
             */

        /** */
        Ext.define("Bar", {});
      EOF
    end

    it "events get extra parameter" do
      events[0][:params].length.should == 2
      events[1][:params].length.should == 1
    end
  end

  describe "Without Ext.define-d class" do
    let(:events) do
      parse(<<-EOF)["Foo"][:members]
        /** @class Foo */
            /**
             * @event click
             * @param {Number} foo
             */
            /**
             * @event touch
             */
      EOF
    end

    it "no extra param gets added" do
      events[0][:params].length.should == 1
      events[1][:params].length.should == 0
    end
  end

end
